@@ -27,6 +27,7 @@ MEDIA_PLAYER_STATE_UNKNOWN = "UNKNOWN"
 TYPE_EDIT_TRACKS_INFO = "EDIT_TRACKS_INFO"
 TYPE_GET_STATUS = "GET_STATUS"
 TYPE_LOAD = "LOAD"
+TYPE_LOAD_FAILED = "LOAD_FAILED"
 TYPE_QUEUE_INSERT = "QUEUE_INSERT"
 TYPE_MEDIA_STATUS = "MEDIA_STATUS"
 TYPE_PAUSE = "PAUSE"
@@ -69,6 +70,36 @@ CMD_SUPPORT_STREAM_TRANSFER = 262144
 CMD_SUPPORT_SKIP_FORWARD = 16
 CMD_SUPPORT_SKIP_BACKWARD = 32
 
+# From https://developers.google.com/cast/docs/web_receiver/error_codes
+MEDIA_PLAYER_ERROR_CODES = {
+    100: "MEDIA_UNKNOWN",
+    101: "MEDIA_ABORTED",
+    102: "MEDIA_DECODE",
+    103: "MEDIA_NETWORK",
+    104: "MEDIA_SRC_NOT_SUPPORTED",
+    110: "SOURCE_BUFFER_FAILURE",
+    201: "MEDIAKEYS_NETWORK",
+    202: "MEDIAKEYS_UNSUPPORTED",
+    203: "MEDIAKEYS_WEBCRYPTO",
+    301: "SEGMENT_NETWORK",
+    311: "HLS_NETWORK_MASTER_PLAYLIST",
+    312: "HLS_NETWORK_PLAYLIST",
+    313: "HLS_NETWORK_NO_KEY_RESPONSE",
+    314: "HLS_NETWORK_KEY_LOAD",
+    315: "HLS_NETWORK_INVALID_SEGMENT",
+    316: "HLS_SEGMENT_PARSING",
+    321: "DASH_NETWORK",
+    322: "DASH_NO_INIT",
+    331: "SMOOTH_NETWORK",
+    332: "SMOOTH_NO_MEDIA_DATA",
+    411: "HLS_MANIFEST_MASTER",
+    412: "HLS_MANIFEST_PLAYLIST",
+    421: "DASH_MANIFEST_NO_PERIODS",
+    422: "DASH_MANIFEST_NO_MIMETYPE",
+    423: "DASH_INVALID_SEGMENT_INFO",
+    431: "SMOOTH_MANIFEST",
+}
+
 
 MediaImage = namedtuple("MediaImage", "url height width")
 
@@ -262,6 +293,8 @@ class MediaStatus:
 
         status_data = data["status"][0]
         media_data = status_data.get("media") or {}
+        if not media_data and "extendedStatus" in status_data:
+            media_data = status_data["extendedStatus"].get("media") or {}
         volume_data = status_data.get("volume", {})
 
         self.current_time = status_data.get("currentTime", self.current_time)
@@ -269,7 +302,8 @@ class MediaStatus:
         self.content_type = media_data.get("contentType", self.content_type)
         self.duration = media_data.get("duration", self.duration)
         self.stream_type = media_data.get("streamType", self.stream_type)
-        self.idle_reason = status_data.get("idleReason", self.idle_reason)
+        # Clear idle reason if not set in the message
+        self.idle_reason = status_data.get("idleReason", None)
         self.media_session_id = status_data.get("mediaSessionId", self.media_session_id)
         self.playback_rate = status_data.get("playbackRate", self.playback_rate)
         self.player_state = status_data.get("playerState", self.player_state)
@@ -317,6 +351,10 @@ class MediaStatusListener(abc.ABC):
     def new_media_status(self, status: MediaStatus):
         """Updated media status."""
 
+    @abc.abstractmethod
+    def load_media_failed(self, item: int, error_code: int):
+        """Called when load media failed."""
+
 
 class BaseMediaPlayer(BaseController):
     """Mixin class for apps which can play media using the default media namespace."""
@@ -503,6 +541,9 @@ class BaseMediaPlayer(BaseController):
         )
         start_play_media_sent.wait(10)
         if not start_play_media_sent.is_set():
+            self.logger.warning(
+                "Quick Play failed for %s:%s(%s)", media_id, media_type, kwargs
+            )
             raise PyChromecastError()
 
 
@@ -533,6 +574,9 @@ class MediaController(BaseMediaPlayer):
         """Called when a media message is received."""
         if data[MESSAGE_TYPE] == TYPE_MEDIA_STATUS:
             self._process_media_status(data)
+            return True
+        if data[MESSAGE_TYPE] == TYPE_LOAD_FAILED:
+            self._process_load_failed(data)
 
             return True
 
@@ -662,7 +706,7 @@ class MediaController(BaseMediaPlayer):
         """Processes a STATUS message."""
         self.status.update(data)
 
-        self.logger.debug("Media:Received status %s", data)
+        self.logger.debug("Media:Updated status %s", self.status)
 
         # Update session active threading event
         if self.status.media_session_id is None:
@@ -672,6 +716,20 @@ class MediaController(BaseMediaPlayer):
 
         self._fire_status_changed()
 
+    def _process_load_failed(self, data):
+        """Processes a LOAD_FAILED message."""
+        item = data.get("itemId")
+        error_code = data.get("detailedErrorCode")
+
+        self.logger.debug(
+            "Media:Load failed with code %s(%s) for item %s",
+            error_code,
+            MEDIA_PLAYER_ERROR_CODES.get(error_code, "unknown code"),
+            item,
+        )
+
+        self._fire_load_failed(item, error_code)
+
     def _fire_status_changed(self):
         """Tells listeners of a changed status."""
         for listener in self._status_listeners:
@@ -680,11 +738,19 @@ class MediaController(BaseMediaPlayer):
             except Exception:  # pylint: disable=broad-except
                 _LOGGER.exception("Exception thrown when calling media status callback")
 
+    def _fire_load_failed(self, item, error_code):
+        """Tells listeners of a changed status."""
+        for listener in self._status_listeners:
+            try:
+                listener.load_media_failed(item, error_code)
+            except Exception:  # pylint: disable=broad-except
+                _LOGGER.exception("Exception thrown when calling load failed callback")
+
     def tear_down(self):
         """Called when controller is destroyed."""
         super().tear_down()
 
-        self._status_listeners[:] = []
+        self._status_listeners = []
 
 
 class DefaultMediaReceiverController(BaseMediaPlayer):
