@@ -17,7 +17,7 @@ import ssl
 import sys
 import threading
 import time
-from collections import namedtuple
+from collections import defaultdict, namedtuple
 from struct import pack, unpack
 
 from . import cast_channel_pb2
@@ -109,7 +109,6 @@ if sys.version_info >= (3, 0):
         """Encodes a python value into JSON format."""
         return json.dumps(data, ensure_ascii=False).encode("utf8")
 
-
 else:
 
     def _json_to_payload(data):
@@ -216,7 +215,7 @@ class SocketClient(threading.Thread):
         self.socket = None
 
         # dict mapping namespace on Controller objects
-        self._handlers = {}
+        self._handlers = defaultdict(set)
         self._connection_listeners = []
 
         self.receiver_controller = ReceiverController(cast_type)
@@ -371,7 +370,11 @@ class SocketClient(threading.Thread):
                             self.port,
                         )
                     return
-                except OSError as err:
+
+                # OSError raised if connecting to the socket fails, NotConnected raised
+                # if another thread tries - and fails - to send a message before the
+                # calls to receiver_controller and heartbeat_controller.
+                except (OSError, NotConnected) as err:
                     self.connecting = True
                     if self.stop.is_set():
                         self.logger.error(
@@ -460,10 +463,20 @@ class SocketClient(threading.Thread):
 
     def register_handler(self, handler: BaseController):
         """Register a new namespace handler."""
-        self._handlers[handler.namespace] = handler
+        self._handlers[handler.namespace].add(handler)
 
         handler.registered(self)
 
+    def unregister_handler(self, handler: BaseController):
+        """Register a new namespace handler."""
+        if (
+            handler.namespace in self._handlers
+            and handler in self._handlers[handler.namespace]
+        ):
+            self._handlers[handler.namespace].remove(handler)
+
+        handler.unregistered()
+
     def new_cast_status(self, cast_status):
         """Called when a new cast status has been received."""
         new_channel = self.destination_id != cast_status.transport_id
@@ -481,7 +494,8 @@ class SocketClient(threading.Thread):
             for namespace in self.app_namespaces:
                 if namespace in self._handlers:
                     self._ensure_channel_connected(self.destination_id)
-                    self._handlers[namespace].channel_connected()
+                    for handler in self._handlers[namespace]:
+                        handler.channel_connected()
 
     def _gen_request_id(self):
         """Generates a unique request id."""
@@ -518,20 +532,19 @@ class SocketClient(threading.Thread):
             return
 
         self.heartbeat_controller.reset()
-        self._force_recon = False
         self.logger.debug("Thread started...")
-        try:
-            while not self.stop.is_set():
+        while not self.stop.is_set():
+            try:
                 if self.run_once(timeout=POLL_TIME_BLOCKING) == 1:
                     break
-        except Exception:  # pylint: disable=broad-except
-            self.logger.exception(
-                ("[%s(%s):%s] Unhandled exception in worker thread"),
-                self.fn or "",
-                self.host,
-                self.port,
-            )
-            raise
+            except Exception:  # pylint: disable=broad-except
+                self._force_recon = True
+                self.logger.exception(
+                    "[%s(%s):%s] Unhandled exception in worker thread, attempting reconnect",
+                    self.fn or "",
+                    self.host,
+                    self.port,
+                )
 
         self.logger.debug("Thread done...")
         # Clean up
@@ -696,32 +709,31 @@ class SocketClient(threading.Thread):
                 )
 
             # message handlers
-            try:
-                handled = self._handlers[message.namespace].receive_message(
-                    message, data
-                )
+            for handler in self._handlers[message.namespace]:
+                try:
+                    handled = handler.receive_message(message, data)
 
-                if not handled:
-                    if data.get(REQUEST_ID) not in self._request_callbacks:
-                        self.logger.debug(
-                            "[%s(%s):%s] Message unhandled: %s",
-                            self.fn or "",
-                            self.host,
-                            self.port,
-                            _message_to_string(message, data),
-                        )
-            except Exception:  # pylint: disable=broad-except
-                self.logger.exception(
-                    (
-                        "[%s(%s):%s] Exception caught while sending message to "
-                        "controller %s: %s"
-                    ),
-                    self.fn or "",
-                    self.host,
-                    self.port,
-                    type(self._handlers[message.namespace]).__name__,
-                    _message_to_string(message, data),
-                )
+                    if not handled:
+                        if data.get(REQUEST_ID) not in self._request_callbacks:
+                            self.logger.debug(
+                                "[%s(%s):%s] Message unhandled: %s",
+                                self.fn or "",
+                                self.host,
+                                self.port,
+                                _message_to_string(message, data),
+                            )
+                except Exception:  # pylint: disable=broad-except
+                    self.logger.exception(
+                        (
+                            "[%s(%s):%s] Exception caught while sending message to "
+                            "controller %s: %s"
+                        ),
+                        self.fn or "",
+                        self.host,
+                        self.port,
+                        type(handler).__name__,
+                        _message_to_string(message, data),
+                    )
 
         else:
             self.logger.debug(
@@ -740,11 +752,12 @@ class SocketClient(threading.Thread):
             except Exception:  # pylint: disable=broad-except
                 pass
 
-        for handler in self._handlers.values():
-            try:
-                handler.tear_down()
-            except Exception:  # pylint: disable=broad-except
-                pass
+        for namespace in self._handlers.values():
+            for handler in namespace:
+                try:
+                    handler.tear_down()
+                except Exception:  # pylint: disable=broad-except
+                    pass
 
         if self.socket is not None:
             try:
@@ -891,12 +904,15 @@ class SocketClient(threading.Thread):
             raise PyChromecastStopped("Socket client's thread is stopped.")
         if not self.connecting and not self._force_recon:
             try:
-                if not no_add_request_id and callback_function:
-                    self._request_callbacks[request_id] = {
-                        "event": threading.Event(),
-                        "response": None,
-                        "function": callback_function,
-                    }
+                if callback_function:
+                    if not no_add_request_id:
+                        self._request_callbacks[request_id] = {
+                            "event": threading.Event(),
+                            "response": None,
+                            "function": callback_function,
+                        }
+                    else:
+                        callback_function(None)
                 self.socket.sendall(be_size + msg.SerializeToString())
             except socket.error:
                 self._request_callbacks.pop(request_id, None)
@@ -908,10 +924,15 @@ class SocketClient(threading.Thread):
                     self.port,
                 )
         else:
-            raise NotConnected("Chromecast {self.host}:{self.port} is connecting...")
+            raise NotConnected(f"Chromecast {self.host}:{self.port} is connecting...")
 
     def send_platform_message(
-        self, namespace, message, inc_session_id=False, callback_function_param=False
+        self,
+        namespace,
+        message,
+        inc_session_id=False,
+        callback_function_param=False,
+        no_add_request_id=False,
     ):
         """Helper method to send a message to the platform."""
         return self.send_message(
@@ -920,10 +941,16 @@ class SocketClient(threading.Thread):
             message,
             inc_session_id,
             callback_function_param,
+            no_add_request_id=no_add_request_id,
         )
 
     def send_app_message(
-        self, namespace, message, inc_session_id=False, callback_function_param=False
+        self,
+        namespace,
+        message,
+        inc_session_id=False,
+        callback_function_param=False,
+        no_add_request_id=False,
     ):
         """Helper method to send a message to current running app."""
         if namespace not in self.app_namespaces:
@@ -938,6 +965,7 @@ class SocketClient(threading.Thread):
             message,
             inc_session_id,
             callback_function_param,
+            no_add_request_id=no_add_request_id,
         )
 
     def register_connection_listener(self, listener: ConnectionStatusListener):
@@ -996,7 +1024,8 @@ class SocketClient(threading.Thread):
         """Handles a channel being disconnected."""
         for namespace in self.app_namespaces:
             if namespace in self._handlers:
-                self._handlers[namespace].channel_disconnected()
+                for handler in self._handlers[namespace]:
+                    handler.channel_disconnected()
 
         self.app_namespaces = []
         self.destination_id = None
